## File : maskedSeq.R
## Function : posMaskedSeq
## Description : this function returns all the positions of the repeated sequences
##
## I.e.
## source("maskedSeq.R")
##
## seq1 = ATGCtgTGTTagtacATTGGGTTTaAAA
## posMaskedSeq("toto","seq1", length = 2)
## Warning message : not all the sequence was masked.
## Number of masked sequences: 2
##      from to
## [1,]    5  6
## [2,]   11 15
##
## posMaskedSeq("promchr22.fasta","chr22_15503251_15505251")
## Number of masked sequences: 1
##      from  to
## [1,]  280 285
##
## posmaskedSeq("promchr22.fasta","chr22_15913504_15915504")
## Number of masked sequences: 0
## NULL
##
## pos <- posMaskedSeq("/net/pizz/All/work/banks/human_genome/build_34_ucsc_chr/chr22.fa",length = 1000000)
## Number of masked sequences: 52344

posMaskedSeqFile <- function(file, name=NA,from=1,to=0, max=10000)
  {
    length <- max
    index <- paste(file,".ix",sep="")
    if (makeIndex(file,type="F")!=-1) {
      if (is.na(name)) {
        name=  as.character(read.table(index,nrows=2)[1,1])
      }
      v <- .paramFromIndex(name, index)
      bS <- v$beginSeq
      size <- v$size
      if (size == -1) {
        return (warning("Error : The sequence doesn't exists in this file"))
      }
      else {
        res <- .C("maskee",
                  file = as.character(file),
                  debSeq = as.integer(bS),
                  tailleSeq = as.integer(size),
                  from = integer(length),
                  to = integer(length),
                  tailleAlloc = as.integer(length),  
                  begSeq = as.integer(from-1),
                  endSeq = as.integer(to),
                  err = integer(1),
                  PACKAGE="GeneR"
                  )
        
        err = res$err
        if (err==1 || err==0) {
          taille =  res$tailleAlloc
          if (taille>=1) {
            from = res$from[1:taille]
            to = res$to[1:taille]
            pos = cbind(from,to)
            if(err==0)
              warning(paste("There are more masked positions than ",max))
            return (pos)
          } else return (NULL)
        } else return(warning("Error : Cannot open the file"))
      }
    }
  }

posMaskedSeq <- function(seqno=0, from=1,to=0,max=10000,type="lower")
  {
    limites <- list(from,to)
                
    liste <- .transf.limites(seqno, limites)
    
    if(liste[[1]] == -1)
      {
        warning("error in the transformation of the limits")
        return (-1)
      }
    from=as.integer(liste[[2]])
    to=as.integer(liste[[3]])
         
    length <- max
    if(to==0)
      to <- sizeSeq()
   if((type=="lower") || type == "upper")
      res <- .C("mask_buffer",
                seqno = as.integer(seqno),
                from = integer(length),
                to = integer(length),
                tailleAlloc = as.integer(length),  
                begSeq = as.integer(from-1),
                endSeq = as.integer(to),
                err = integer(1),
                PACKAGE="GeneR"
                )
    else
      res <- .C("mask_buffer_char",
                seqno = as.integer(seqno),
                from = integer(length),
                to = integer(length),
                tailleAlloc = as.integer(length),
                caracter = as.character(type),
                begSeq = as.integer(from-1),
                endSeq = as.integer(to),
                err = integer(1),
                PACKAGE="GeneR"
                )
     
        
        err = res$err
        if (err==1 || err==0) {
          taille =  res$tailleAlloc
          if (taille>=1) {
            from = res$from[1:taille]
            to = res$to[1:taille]
            pos = cbind(from,to)
            if(type=="upper")
              {
                pos <- cbind(from-1,to+1)
                pos <- xorRecouvr(pos,c(1,sizeSeq(seqno)))
                
              }
            if(err==0)
              warning(paste("There are more masked positions than ",max))
            
            return (pos+getBegSeq(seqno)-1)
          }
          else
            {
              # We have not found any regions in lower case
              if(type=="upper")
                return(cbind(1,sizeSeq(seqno)))
              else
                return (numeric(0))
            }
        } else return(warning("Error : Cannot open the file"))
  }
    
  



upperSeq <- function(seqno=0, from=1, to=0)
  {
    limites <- list(from,to)
                
    liste <- .transf.limites(seqno, limites)
    
    if(liste[[1]] == -1)
      {
        warning("error in the transformation of the limits")
        return (-1)
      }
    from=as.integer(liste[[2]])
    to=as.integer(liste[[3]])
         
    n = length(from)
    if(length(to) != n)
      {
        warning("Variable from and to must be of equal length")
        return(-1)
      }
    .C("upper_buffer",
       seqno=as.integer(seqno),
       from=as.integer(from),
       to=as.integer(to),
       reverse=as.integer(0),
       longfrom=as.integer(n),
       PACKAGE="GeneR"
       )

    if(.seqSize()$complementaire[seqno+1]>0)
      {
        lestrand <- getStrand()
        setStrand(1)
        limites <- list(from,to)
        
        
        liste <- .transf.limites(seqno, limites)

        if(liste[[1]] == -1)
          {
            warning("error in the transformation of the limits")
            return (-1)
          }

        .C("upper_buffer",
           seqno=as.integer(seqno),
           from=as.integer(liste[[2]]),
           to=as.integer(liste[[3]]),
           reverse=as.integer(1),
           longfrom=as.integer(n),
           PACKAGE="GeneR"
           )
        
        setStrand(lestrand)
      }

    return(1)
  }



lowerSeq <- function(seqno=0, from=1, to=0)
  {
    limites <- list(from,to)
                
    liste <- .transf.limites(seqno, limites)
    
    if(liste[[1]] == -1)
      {
        warning("error in the transformation of the limits")
        return (-1)
      }
    from=as.integer(liste[[2]])
    to=as.integer(liste[[3]])
         
    n = length(from)
    if(length(to) != n)
      {
        warning("Variable from and to must be of equal length")
        return(-1)
      }
    .C("lower_buffer",
       seqno=as.integer(seqno),
       from=as.integer(from),
       to=as.integer(to),
       reverse=as.integer(0),
       longfrom=as.integer(n),
       PACKAGE="GeneR"
       )

    if(.seqSize()$complementaire[seqno+1]>0)
      {
        lestrand <- getStrand()
        setStrand(1)
        limites <- list(from,to)
        
        
        liste <- .transf.limites(seqno, limites)

        if(liste[[1]] == -1)
          {
            warning("error in the transformation of the limits")
            return (-1)
          }

        .C("lower_buffer",
           seqno=as.integer(seqno),
           from=as.integer(liste[[2]]),
           to=as.integer(liste[[3]]),
           reverse=as.integer(1),
           longfrom=as.integer(n),
           PACKAGE="GeneR"
           )
        
        setStrand(lestrand)
      }


    return(1)
  }


mask <- function(seqno=0,from=1,to=0,letter='N')
  {
    lestrand <- getStrand()
    setStrand(0)
    limites <- list(from,to)
    liste <- .transf.limites(seqno, limites)

    from <- liste[[2]]
    to <- liste[[3]]
    setStrand(lestrand)
    
    if(liste[[1]] == -1)
      {
        warning("error in the transformation of the limits")
        return (NULL)
      }

    
    
    res<-.C("mask",
            as.integer(seqno),
            as.integer(from),
            as.integer(to),
            as.integer(length(to)),
            as.character(letter),
            err=as.integer(1),
            PACKAGE="GeneR"
            )$err
    if(res == -1)
      return(NULL)
    else
      1
    
  }
