/*! \file GeneR_glob.cc
 *  \brief Global variables of GeneR (i.e sequences buffers)
 *   The global variable myGeneR is allocated here, and access trough
 *  functions defined here too. The global variable myGeneR is a vector
 *  of elements of class GeneR_seq.
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-03-06 17:43:01 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include "GeneR_seq.h"
#include <stdio.h>
#include <vector>
//#include "GeneR_glob.h"
#include "complementaire.h"


extern "C"
{
  void init_GeneR(int * maxbuffers);
  void myGeneR_alloc (int *size, int *bufno, int *fill, int *comp,int * err);
  void size_buf (int  *bufno,int * comp);
  void sys_maxbuffer(int * nb);
  void getParam(int * bufno, int * beg, int * size, int * masterSize, int * err);
  void setParam(int * bufno, int * beg, int * masterSize, int * err);
  void freeSeq(int * bufno);
  void getAccno (int * bufno,char ** name,int * err);
  void setAccno (int * bufno,char ** name,int * err);
  void getStrand (int * strandR);
  void setStrand (int * strandR);

}
/* THE GLOBAL VARIABLE */
/* MAX_BUFFERS is initialised with function init_GeneR
 * call from zzz.R file 
 */

using namespace std;
int max_buffers = 0;
//static GeneR_seq * myGeneR;
static vector<GeneR_seq> myGeneR;
int strand=0;


/** \var max_buffers
 * \brief global variable of maximum buffer allowed
 * \note this variable can be increase using init_GeneR.
 */
/** \var myGeneR
 * \brief global variable containing all sequences buffers.
 * myGeneR[0] is an element of class GeneR_seq that contains
 * sequence, complementaire sequence and parameters sSize, sSizeComp
 * BegSeq and MasterSize
 */
/** \var strand
 * \brief global variable of current strand used
 */

/** init_GeneR(int * maxbuffers)
 * \brief initialize all buffer, maxbuffers is a parameter, can
 *        be use to increase maximum buffers
 * \param maxbuffers number of buffers 
 */
void init_GeneR(int * maxbuffers)
{


  extern vector<GeneR_seq> myGeneR;

  myGeneR.resize(*maxbuffers);

  max_buffers = *maxbuffers;

}


/** \fn myGeneRbuffer(int bufno,int comp=0)
 * \brief return the sequence of a specific buffer; NULL if bufno
 *  not in range.
 * \param bufno buffer number 
 * \param comp 0: forward, 1 reverse.
 */
char * myGeneRbuffer(int bufno,int comp=0)
{
  extern vector<GeneR_seq> myGeneR;

  if((bufno <0) || (bufno > max_buffers))
    return(NULL);
  if(comp)
    {
      if(myGeneR[bufno].sSize != myGeneR[bufno].sSizeComp)
	complementaire::buf_sys_complementaire(bufno);
      return(myGeneR[bufno].seqComp);

    }
  else
    return(myGeneR[bufno].seq);
}


/** \fn myGeneRsize(int bufno,int comp=0)
 * \brief returns size of a buffer. Returns -1 if buffer not in range
 * \param bufno buffer number 
 * \param comp 0: forward, 1 reverse.
 * \note size of buffer uninitialized is 0;
*/
int myGeneRsize(int bufno,int comp=0)
{
  if((bufno <0) || (bufno > max_buffers))
    return(-1);

  if(comp)
    return(myGeneR[bufno].sSizeComp);
  else
    return(myGeneR[bufno].sSize);
}

/** \fn myGeneRmaxBuffers()
 * \brief returns global variable max_buffers
 */
int myGeneRmaxBuffers()
{
  return(max_buffers);
}

/** \fn sys_maxbuffer(int * nb)
 * \brief returns global variable max_buffers
 */
void sys_maxbuffer(int * nb)
{
  *nb= max_buffers;
}

/** \fn myGeneRallocBuffer(int size, int bufno, int comp, int fill )
 * \brief (re) allocate a buffer. fill=1: add a "\0" at position 0
 *  returns 0 if error; 1: OK.
 * \param bufno buffer number 
 * \param comp 0: forward, 1 reverse.
 * \param size size of buffer (including "\0")
 * \param fill fill=1: add a '\0' at begining of sequence
 */
int myGeneRallocBuffer(int size, int bufno, int comp, int fill )
{
  if((bufno <0) || (bufno > max_buffers))
    return(0);
      
  return(myGeneR[bufno].allocBuffer(size, comp, fill));
}




/** @brief allocate a buffer
    @param *size  Taille a allouer
    @param *bufno Numero du bufseq
    @param *fill  1: add a "\0" at begining
    @param *comp  0 : brin sens -  1 : brin anti-sens
    @param *err 0: error, 1: good
*/
/* ---------------------------------------------------- */
void myGeneR_alloc (int *size, int *bufno, int *fill, int *comp,int * err)
{
  if((*bufno <0) || (*bufno > max_buffers))
    {
      *err = 0;
      return;
    }
  *err = myGeneR[*bufno].allocBuffer(*size, *comp, *fill);
}



/** @brief size of buffer

   return length of sequence in bufno.   
   @param  *bufno Integer, number of the sequence. (bufseq)
   @param  *comp  0 : brin sens -  1 : brin anti-sens)
*/
void  size_buf (int  *bufno,int * comp)
{
  if((*bufno <0) || (*bufno > max_buffers))
    {
      *bufno = -1;
      return;
    }
  if(*comp)
    *bufno = myGeneR[*bufno].sSizeComp;
  else
    *bufno = myGeneR[*bufno].sSize;

}


/** \fn getParam(int * bufno, int * beg, int * size, int * masterSize, int * err)
 * \brief get globals parameters of a sequence
 * \param bufno buffer sequence
 * \param beg begining position on source sequence
 * \param size size of sequence (not including "\0")
 * \param masterSize size of source sequence
 * \param err 0: is buffno not in range 1 good
 */
void getParam(int * bufno, int * beg, int * size, int * masterSize, int * err)
{
   if((*bufno <0) || (*bufno > max_buffers))
    {
      *err = 0;
      return;
    }
   *err = 1;
   *beg = myGeneR[*bufno].BegSeq;
   *masterSize = myGeneR[*bufno].MasterSize;
   *size = myGeneR[*bufno].sSize - 1;
 
}

/** \fn setParam(int * bufno, int * beg, int * masterSize, int * err)
 * \brief set globals parameters of a sequence
 * \param bufno buffer sequence
 * \param beg begining position on source sequence
 * \param masterSize size of source sequence
 * \param err 0: is buffno not in range 1 good
 */
void setParam(int * bufno, int * beg, int * masterSize, int * err)
{
   if((*bufno <0) || (*bufno > max_buffers))
    {
      *err = 0;
      return;
    }
   *err = 1;
   myGeneR[*bufno].BegSeq= *beg;
   myGeneR[*bufno].MasterSize=*masterSize;
   /*   myGeneR[*bufno].SeqSize=*size;*/
}

/** \fn freeSeq(int * bufno)
 * \brief free sequence and complementaire
 * \param bufno sequence number
 * \note set bufno to 0 is error (i.e. buffer out of range)
 */
void freeSeq(int * bufno)
{
  if((*bufno <0) || (*bufno > max_buffers))
    {
      *bufno = 0;
      return;
    }
  myGeneR[*bufno].freeSeq();

}

/** \fn GeneRfreeSeq(int  bufno)
 * \brief free sequence and complementaire
 * \param bufno sequence number
 * \note set bufno to 0 is error (i.e. buffer out of range)
 */
void GeneRfreeSeq(int  bufno)
{
  if((bufno <0) || (bufno > max_buffers))
    {
      bufno = 0;
      return;
    }
  myGeneR[bufno].freeSeq();

}

/** \fn GeneRfreeCompSeq(int  bufno)
 * \brief free sequence complementaire
 * \param bufno sequence number
 * \note set bufno to 0 is error (i.e. buffer out of range)
 */
void GeneRfreeCompSeq(int  bufno)
{
  if((bufno <0) || (bufno > max_buffers))
    {
      bufno = 0;
      return;
    }
  myGeneR[bufno].freeCompSeq();

}

/** \fn setStrand  (int * strandR)
 * \brief set global variable "strand"
 */
void setStrand (int * strandR)
{
  strand = strandR[0];
}

/** \fn getStrand (int * strandR)
 * \brief get global variable "strand"
 */
void getStrand (int * strandR)
{
  strandR[0] = strand;
}

/** \fn  getAccno(int * bufno,char ** name,int * err)
 * \brief get accNo of a sequence
 * \param bufno number of sequence
 * \param name name of sequence
 * \param err: 0 if buffno out of range else 1
 */
void getAccno (int * bufno,char ** name,int * err)
{
   if((*bufno <0) || (*bufno > max_buffers))
    {
      *err = 0;
      return;
    }
   strncpy(name[0],myGeneR[*bufno].AccN,MaxLNom);
   *err=1;
}

/** \fn  setAccno(int * bufno,char ** name,int * err)
 * \brief set accNo of a sequence
 * \param bufno number of sequence
 * \param name name of sequence
 * \param err: 0 if buffno out of range else 1
 */
void setAccno (int * bufno,char ** name,int * err)
{
   if((*bufno <0) || (*bufno > max_buffers))
    {
      *err = 0;
      return;
    }
   strncpy(myGeneR[*bufno].AccN,name[0],MaxLNom);
   *err=1;
}
